from typing import List

from src.jptranstokenizer.mainword.juman import JumanTokenizer
from src.jptranstokenizer.mainword.spacy_luw import SpacyluwTokenizer
from src.jptranstokenizer.mainword.sudachi import SudachiTokenizer

sentence_1: str = "未来科学部でコンビニ店員になりきってお釣りを返していこう！"
sentence_2: str = "外国人参政権"
sentence_3: str = "魔法少女リリカルなのは"


def test_juman() -> None:
    tokenizer: JumanTokenizer = JumanTokenizer()
    lst_tokens_1: List[str] = [
        "未来",
        "科学",
        "部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なりきって",
        "お釣り",
        "を",
        "返して",
        "いこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国", "人", "参政", "権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リリカルなのは"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_spacyluw() -> None:
    tokenizer: SpacyluwTokenizer = SpacyluwTokenizer()
    lst_tokens_1: List[str] = [
        "未来科学部",
        "で",
        "コンビニ店員",
        "に",
        "なりきっ",
        "て",
        "お釣り",
        "を",
        "返し",
        "ていこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国人参政権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法少女リリカル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_sudachi_a() -> None:
    tokenizer: SudachiTokenizer = SudachiTokenizer(split_mode="A")
    lst_tokens_1: List[str] = [
        "未来",
        "科学",
        "部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なり",
        "きっ",
        "て",
        "お",
        "釣り",
        "を",
        "返し",
        "て",
        "いこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国", "人", "参政", "権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リリカル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_sudachi_b() -> None:
    tokenizer: SudachiTokenizer = SudachiTokenizer(split_mode="B")
    lst_tokens_1: List[str] = [
        "未来",
        "科学部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なり",
        "きっ",
        "て",
        "お釣り",
        "を",
        "返し",
        "て",
        "いこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国人", "参政権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リリカル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_sudachi_c() -> None:
    tokenizer: SudachiTokenizer = SudachiTokenizer(split_mode="C")
    lst_tokens_1: List[str] = [
        "未来科学部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なり",
        "きっ",
        "て",
        "お釣り",
        "を",
        "返し",
        "て",
        "いこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国人参政権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リリカル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3



import unicodedata
from abc import ABC, abstractmethod
from typing import List, Optional


class MainTokenizerABC(ABC):
    def __init__(
        self,
        do_lower_case: bool = False,
        never_split: Optional[List[str]] = None,
        normalize_text: bool = True,
        **kwargs
    ) -> None:
        """
        Abstract main word tokenizer.

        Parameters
        ----------
            **do_lower_case**: (*optional*) boolean (default True)
                Whether to lowercase the input.
            **never_split**: (*optional*) list of str
                Kept for backward compatibility purposes. Now implemented directly
                at the base class level (see
                [`PreTrainedTokenizer.tokenize`]) List of tokens not to split.
            **normalize_text**: (*optional*) boolean (default True)
                Whether to apply unicode normalization to text before tokenization.
        """
        self.do_lower_case = do_lower_case
        self.never_split = never_split if never_split is not None else []
        self.normalize_text = normalize_text

    @abstractmethod
    def tokenize(
        self, text: str, never_split: Optional[List[str]] = None, **kwargs
    ) -> List[str]:
        pass


class Normalizer(MainTokenizerABC):
    def __init__(
        self,
        do_lower_case: bool = False,
        never_split: Optional[List[str]] = None,
        normalize_text: bool = True,
    ):
        super().__init__(do_lower_case=False, never_split=None, normalize_text=True)

    def tokenize(self, text: str, never_split: Optional[List[str]] = None) -> List[str]:
        """Tokenizes a piece of text."""
        if self.normalize_text:
            text = unicodedata.normalize("NFKC", text)
        never_split = self.never_split + (
            never_split if never_split is not None else []
        )
        return text



import unicodedata
from typing import List, Optional

from .base import MainTokenizerABC


try:
    import spacy
except ModuleNotFoundError as error:
    raise error.__class__(
        "You need to install spacy to use SpacyluwTokenizer.\n"
        "See https://pypi.org/project/spacy/ for spacy installation.\n"
        "Also you would need to install ja_gsdluw "
        "https://github.com/megagonlabs/UD_Japanese-GSD/releases/tag/r2.9-NE "
        "for ja_gsdluw installation."
    )
except OSError as error:
    raise error.__class__(
        "You need to install ja_gsdluw to use SpacyluwTokenizer.\n"
        "See https://github.com/megagonlabs/UD_Japanese-GSD/releases/tag/r2.9-NE "
        "for installation."
    )


class SpacyluwTokenizer(MainTokenizerABC):
    def __init__(
        self,
        do_lower_case: bool = False,
        never_split: Optional[List[str]] = None,
        normalize_text: bool = True,
    ):
        super().__init__(do_lower_case=False, never_split=None, normalize_text=True)
        self.nlp = spacy.load("ja_gsdluw")

    def tokenize(self, text: str, never_split: Optional[List[str]] = None) -> List[str]:
        if self.normalize_text:
            text = unicodedata.normalize("NFKC", text)

        never_split = self.never_split + (
            never_split if never_split is not None else []
        )
        tokens = []
        doc = self.nlp(text)
        tokens = [token.text for sent in doc.sents for token in sent]
        if self.do_lower_case:
            tokens = [
                token if token in never_split else token.lower() for token in tokens
            ]
        return tokens



import collections
import os

from typing import Dict, List, Optional, Union

import transformers
from transformers import (
    AlbertTokenizer,
    BertJapaneseTokenizer,
    BertTokenizer,
    logging,
)
from transformers.models.bert.tokenization_bert import (
    BasicTokenizer,
    WordpieceTokenizer,
)
from transformers.models.bert_japanese.tokenization_bert_japanese import (
    CharacterTokenizer,
    MecabTokenizer,
)


logging.set_verbosity_info()
logging.enable_explicit_format()
logger = logging.get_logger()

PUBLIC_AVAILABLE_SETTING_MAP: Dict[str, Dict[str, str]] = {
    "cl-tohoku/bert-base-japanese": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "ipadic",
    },
    "cl-tohoku/bert-base-japanese-v2": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "unidic_lite",
    },
    "cl-tohoku/bert-base-japanese-whole-word-masking": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "ipadic",
    },
    "cl-tohoku/bert-large-japanese": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "unidic_lite",
    },
    "ken11/albert-base-japanese-v1-with-japanese-tokenizer": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "ipadic",
    },
    "nlp-waseda/roberta-base-japanese": {
        "word_tokenizer": "juman",
        "tokenizer_class": "AlbertTokenizer",
    },
    "nlp-waseda/roberta-large-japanese": {
        "word_tokenizer": "juman",
        "tokenizer_class": "AlbertTokenizer",
    },
    "nlp-waseda/roberta-large-japanese-seq512": {
        "word_tokenizer": "juman",
        "tokenizer_class": "AlbertTokenizer",
    },
    "rinna/japanese-roberta-base": {
        "do_word_tokenize": False,
        "word_tokenizer": "",
        "tokenizer_class": "T5Tokenizer",
    },
}

IZUMILAB_SETTING_MAP: Dict[str, Dict[str, str]] = {
    f"izumi-lab/{model_name}": {
        "word_tokenizer": "mecab",
        "tokenizer_class": "BertJapaneseTokenizer",
        "mecab_dic": "ipadic",
    }
    for model_name in [
        "bert-small-japanese",
        "bert-small-japanese-fin",
        "electra-base-japanese-discriminator",
        "electra-base-japanese-generator",
        "electra-small-japanese-discriminator",
        "electra-small-japanese-fin-discriminator",
        "electra-small-japanese-fin-generator",
        "electra-small-japanese-generator",
        "electra-small-paper-japanese-discriminator",
        "electra-small-paper-japanese-fin-discriminator",
        "electra-small-paper-japanese-fin-generator",
        "electra-small-paper-japanese-generator",
    ]
}

PUBLIC_AVAILABLE_SETTING_MAP.update(IZUMILAB_SETTING_MAP)


def get_word_tokenizer(
    word_tokenizer_type: str,
    do_lower_case: bool,
    never_split: Optional[List[str]] = None,
    normalize_text: bool = True,
    mecab_dic: Optional[str] = "ipadic",
    mecab_option: Optional[str] = None,
    sudachi_split_mode: Optional[str] = "A",
    sudachi_config_path: Optional[str] = None,
    sudachi_resource_dir: Optional[str] = None,
    sudachi_dict_type: Optional[str] = "core",
):
    if word_tokenizer_type == "basic":
        logger.warn("Argument normalize_text is ignored")
        word_tokenizer = BasicTokenizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            tokenize_chinese_chars=False,
        )
    elif word_tokenizer_type == "mecab":
        word_tokenizer = MecabTokenizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            normalize_text=normalize_text,
            mecab_dic=mecab_dic,
            mecab_option=mecab_option,
        )
    elif word_tokenizer_type == "juman":
        from .mainword.juman import JumanTokenizer

        word_tokenizer = JumanTokenizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            normalize_text=normalize_text,
        )
    elif word_tokenizer_type == "spacy-luw":
        from .mainword.spacy_luw import SpacyluwTokenizer

        word_tokenizer = SpacyluwTokenizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            normalize_text=normalize_text,
        )
    elif word_tokenizer_type == "sudachi":
        from .mainword.sudachi import SudachiTokenizer

        word_tokenizer = SudachiTokenizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            normalize_text=normalize_text,
            split_mode=sudachi_split_mode,
            config_path=sudachi_config_path,
            resource_dir=sudachi_resource_dir,
            dict_type=sudachi_dict_type,
        )
    elif word_tokenizer_type == "none":
        from .mainword.base import Normalizer

        word_tokenizer = Normalizer(
            do_lower_case=do_lower_case,
            never_split=never_split,
            normalize_text=normalize_text,
        )
    else:
        raise ValueError(
            f"Invalid word_tokenizer_type '{word_tokenizer_type}' is specified."
        )
    return word_tokenizer


class JapaneseTransformerTokenizer(BertJapaneseTokenizer):
    def __init__(
        self,
        vocab_file="",
        do_lower_case=False,
        do_word_tokenize=True,
        do_subword_tokenize=True,
        word_tokenizer="basic",
        subword_tokenizer="wordpiece",
        never_split=None,
        unk_token="[UNK]",
        sep_token="[SEP]",
        pad_token="[PAD]",
        cls_token="[CLS]",
        mask_token="[MASK]",
        call_from_pretrained=False,
        mecab_dic: Optional[str] = "ipadic",
        mecab_option: Optional[str] = None,
        sudachi_split_mode: Optional[str] = "A",
        sudachi_config_path: Optional[str] = None,
        sudachi_resource_dir: Optional[str] = None,
        sudachi_dict_type: Optional[str] = "core",
        sp_model_kwargs: Optional[str] = None,
        **kwargs,
    ):
        super(BertTokenizer, self).__init__(
            unk_token=unk_token,
            sep_token=sep_token,
            pad_token=pad_token,
            cls_token=cls_token,
            mask_token=mask_token,
            do_lower_case=do_lower_case,
            do_word_tokenize=do_word_tokenize,
            do_subword_tokenize=do_subword_tokenize,
            word_tokenizer=word_tokenizer,
            subword_tokenizer=subword_tokenizer,
            never_split=never_split,
            **kwargs,
        )
        # ^^ We call the grandparent's init, not the parent's.

        if not os.path.isfile(vocab_file) and not call_from_pretrained:
            raise ValueError(
                f"Can't find a vocabulary file at path '{vocab_file}'.\n"
                "To load the vocabulary from a Google pretrained model use "
                "`AutoTokenizer.from_pretrained(PRETRAINED_MODEL_NAME)`"
            )
        # if subword_tokenizer != "sentencepiece" and not call_from_pretrained:
        #     self.vocab = load_vocab(vocab_file)
        #     self.ids_to_tokens = collections.OrderedDict(
        #         [(ids, tok) for tok, ids in self.vocab.items()]
        #     )

        self.do_word_tokenize = do_word_tokenize
        self.lower_case = do_lower_case
        self.never_split = never_split
        if do_word_tokenize:
            self.word_tokenizer = get_word_tokenizer(
                word_tokenizer_type=word_tokenizer,
                do_lower_case=do_lower_case,
                never_split=never_split,
                mecab_dic=mecab_dic,
                mecab_option=mecab_option,
                sudachi_split_mode=sudachi_split_mode,
                sudachi_config_path=sudachi_config_path,
                sudachi_resource_dir=sudachi_resource_dir,
                sudachi_dict_type=sudachi_dict_type,
            )

        self.do_subword_tokenize = do_subword_tokenize
        self.subword_tokenizer_type = subword_tokenizer
        if self.do_subword_tokenize and not call_from_pretrained:
            if self.subword_tokenizer_type == "wordpiece":
                self.subword_tokenizer = WordpieceTokenizer(
                    vocab=self.vocab, unk_token=self.unk_token
                )
            elif self.subword_tokenizer_type == "character":
                self.subword_tokenizer = CharacterTokenizer(
                    vocab=self.vocab, unk_token=self.unk_token
                )
            elif self.subword_tokenizer_type == "sentencepiece":
                from subword import SentencePieceTokenizer

                self.subword_tokenizer = SentencePieceTokenizer(
                    vocab_file=vocab_file, sp_model_kwargs=sp_model_kwargs
                )
                self.vocab = self.subword_tokenizer.vocab
                self.ids_to_tokens = collections.OrderedDict(
                    [
                        (i, self.subword_tokenizer.spm.IdToPiece(i))
                        for i in range(self.subword_tokenizer.bpe_vocab_size)
                    ]
                )
            else:
                raise ValueError(
                    f"Invalid subword_tokenizer '{subword_tokenizer}' is specified."
                )
        # This is needed for leave special tokens as it is when tokenizing
        self.unique_no_split_tokens = list(self.special_tokens_map.values())
        if self.subword_tokenizer_type == "sentencepiece":
            self.save_vocabulary = AlbertTokenizer.save_vocabulary

    @classmethod
    def from_pretrained(cls, tokenizer_name_or_path: Union[str, os.PathLike], **kwargs):
        def _from_pretrained(
            word_tokenizer: Optional[str] = None,
            tokenizer_class: Optional[str] = None,
            do_lower_case: bool = False,
            do_word_tokenize: bool = True,
            never_split: bool = None,
            mecab_dic: Optional[str] = "ipadic",
            mecab_option: Optional[str] = None,
            sudachi_split_mode: Optional[str] = "A",
            sudachi_config_path: Optional[str] = None,
            sudachi_resource_dir: Optional[str] = None,
            sudachi_dict_type: Optional[str] = "core",
            sp_model_kwargs: Optional[str] = None,
            *init_inputs,
            **kwargs,
        ):

            tokenizer_class = (
                transformers.models.auto.tokenization_auto.tokenizer_class_from_name(
                    tokenizer_class
                )
            )
            tentative_tokenizer = tokenizer_class.from_pretrained(
                tokenizer_name_or_path, *init_inputs, **kwargs
            )
            if isinstance(tentative_tokenizer, transformers.T5Tokenizer) or isinstance(
                tentative_tokenizer, transformers.AlbertTokenizer
            ):
                # sentencepiece
                subword_tokenizer_type = "sentencepiece"
                from .subword.sentencepiece import SentencePieceTokenizer

                subword_tokenizer = SentencePieceTokenizer(
                    vocab_file=None,
                    sp_model_kwargs=sp_model_kwargs,
                    spm=tentative_tokenizer.sp_model,
                )
                vocab = subword_tokenizer.vocab
                ids_to_tokens = collections.OrderedDict(
                    [
                        (i, subword_tokenizer.spm.IdToPiece(i))
                        for i in range(subword_tokenizer.bpe_vocab_size)
                    ]
                )
            elif isinstance(tentative_tokenizer, BertJapaneseTokenizer):
                # WordPiece or character
                subword_tokenizer = tentative_tokenizer.subword_tokenizer
                if isinstance(subword_tokenizer, WordpieceTokenizer):
                    subword_tokenizer_type = "wordpiece"
                elif isinstance(subword_tokenizer, CharacterTokenizer):
                    subword_tokenizer_type = "character"
                else:
                    raise ValueError()
                vocab = tentative_tokenizer.vocab
                ids_to_tokens = tentative_tokenizer.ids_to_tokens
            else:
                raise NotImplementedError()
            tokenizer = cls(
                do_lower_case=do_lower_case,
                do_word_tokenize=do_word_tokenize,
                do_subword_tokenize=True,
                word_tokenizer=word_tokenizer,
                subword_tokenizer=subword_tokenizer_type,
                never_split=never_split,
                unk_token=tentative_tokenizer.special_tokens_map["unk_token"],
                sep_token=tentative_tokenizer.special_tokens_map["sep_token"],
                pad_token=tentative_tokenizer.special_tokens_map["pad_token"],
                cls_token=tentative_tokenizer.special_tokens_map["cls_token"],
                mask_token=tentative_tokenizer.special_tokens_map["mask_token"],
                call_from_pretrained=True,
                mecab_dic=mecab_dic,
                mecab_option=mecab_option,
                sudachi_split_mode=sudachi_split_mode,
                sudachi_config_path=sudachi_config_path,
                sudachi_resource_dir=sudachi_resource_dir,
                sudachi_dict_type=sudachi_dict_type,
            )
            tokenizer.subword_tokenizer = subword_tokenizer
            tokenizer.vocab = vocab
            tokenizer.ids_to_tokens = ids_to_tokens

            # This is needed for leave special tokens as it is when tokenizing
            tokenizer.unique_no_split_tokens = list(
                tokenizer.special_tokens_map.values()
            )
            return tokenizer

        if tokenizer_name_or_path in [
            "megagonlabs/electra-base-japanese-discriminator",
            "megagonlabs/transformers-ud-japanese-electra-base-discriminator",
        ]:
            raise NotImplementedError(
                (
                    f"Loading {tokenizer_name_or_path} is not expected in this module.\n"
                    "Please use the official implementation."
                )
            )

        if tokenizer_name_or_path in PUBLIC_AVAILABLE_SETTING_MAP.keys():
            dct_setting: Dict[str, str] = PUBLIC_AVAILABLE_SETTING_MAP[
                tokenizer_name_or_path
            ]
            for k, v in dct_setting.items():
                kwargs[k] = v
        else:
            if kwargs["word_tokenizer"] is None:
                raise ValueError("word_tokenizer must be specified")
            if kwargs["tokenizer_class"] is None:
                raise ValueError("tokenizer_class must be specified")
        return _from_pretrained(**kwargs)

    def convert_tokens_to_string(self, tokens):
        if self.subword_tokenizer_type in ["character", "wordpiece"]:
            return super().convert_tokens_to_string(self, tokens)
        elif self.subword_tokenizer_type == "sentencepiece":
            return self.subword_tokenizer.spm.decode(tokens)
        else:
            raise NotImplementedError(
                f"{self.subword_tokenizer} is not allowed for convert_tokens_to_string"
            )



from typing import List


class SentencePieceTokenizer:
    def __init__(self, vocab_file, sp_model_kwargs, spm=None):
        if spm is None:
            import sentencepiece as sp

            self.sp_model_kwargs = {} if sp_model_kwargs is None else sp_model_kwargs
            self.spm = sp.SentencePieceProcessor(**self.sp_model_kwargs)
            self.spm.load(vocab_file)
        else:
            self.spm = spm
        self.bpe_vocab_size = self.spm.GetPieceSize()
        self.vocab = {self.spm.IdToPiece(i): i for i in range(self.bpe_vocab_size)}

    def tokenize(
        self,
        text: str,
    ) -> List[str]:
        tokens = self.spm.encode_as_pieces(text)
        return tokens



import unicodedata
from typing import List, Optional

from .base import MainTokenizerABC

try:
    from sudachitra.sudachipy_word_tokenizer import SudachipyWordTokenizer
    from sudachitra.word_formatter import word_formatter
except ModuleNotFoundError as error:
    raise error.__class__(
        "You need to install sudachitra to use SudachipyWordTokenizer."
        "See https://pypi.org/project/SudachiTra/ for installation."
    )
    # cf. https://pypi.org/project/SudachiTra/
    # cf. https://github.com/WorksApplications/SudachiTra/blob/main/sudachitra/tokenization_bert_sudachipy.py
    # cf. https://github.com/WorksApplications/SudachiTra/blob/main/sudachitra/sudachipy_word_tokenizer.py


class SudachiTokenizer(MainTokenizerABC):
    def __init__(
        self,
        do_lower_case: bool = False,
        never_split: Optional[List[str]] = None,
        normalize_text: bool = True,
        split_mode: Optional[str] = "A",
        config_path: Optional[str] = None,
        resource_dir: Optional[str] = None,
        dict_type: Optional[str] = "core",
    ):
        super().__init__(do_lower_case=False, never_split=None, normalize_text=True)
        self.sudachi_tokenizer = SudachipyWordTokenizer(
            split_mode=split_mode,
            config_path=config_path,
            resource_dir=resource_dir,
            dict_type=dict_type,
        )
        self.word_formatter = word_formatter(
            "surface", self.sudachi_tokenizer.sudachi_dict
        )

    def tokenize(self, text: str, never_split: Optional[List[str]] = None) -> List[str]:
        if self.normalize_text:
            text = unicodedata.normalize("NFKC", text)

        never_split = self.never_split + (
            never_split if never_split is not None else []
        )
        tokens = [
            self.word_formatter(token)
            for token in self.sudachi_tokenizer.tokenize(text)
        ]
        if self.do_lower_case:
            tokens = [
                token if token in never_split else token.lower() for token in tokens
            ]
        return tokens



from typing import List

from src.jptranstokenizer import JapaneseTransformerTokenizer

sentence_1: str = "未来科学部でコンビニ店員になりきってお釣りを返していこう！"
sentence_2: str = "外国人参政権"
sentence_3: str = "魔法少女リリカルなのは"


# @pytest.mark.parametrize("model_name", list(tku.PUBLIC_AVAILABLE_SETTING_MAP.keys()))
# def test_public_tokenizer(model_name: str) -> None:
#     tku.JapaneseTransformerTokenizer.from_pretrained(model_name)


def test_cltohoku_bertbasejapanese() -> None:
    tokenizer: JapaneseTransformerTokenizer = (
        JapaneseTransformerTokenizer.from_pretrained("cl-tohoku/bert-base-japanese")
    )
    lst_tokens_1: List[str] = [
        "未来",
        "科学",
        "部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なり",
        "きっ",
        "て",
        "お",
        "##釣",
        "##り",
        "を",
        "返し",
        "て",
        "いこ",
        "う",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国", "人", "##参", "政権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リ", "##リカル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_cltohoku_bertbasejapanese_v2() -> None:
    tokenizer: JapaneseTransformerTokenizer = (
        JapaneseTransformerTokenizer.from_pretrained("cl-tohoku/bert-base-japanese-v2")
    )
    lst_tokens_1: List[str] = [
        "未来",
        "科学",
        "部",
        "で",
        "コンビニ",
        "店員",
        "に",
        "なり",
        "きっ",
        "て",
        "お",
        "釣り",
        "を",
        "返し",
        "て",
        "いこう",
        "!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["外国", "人", "##参", "政権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["魔法", "少女", "リリ", "##カル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3


def test_nlpwaseda_robertabasejapanese() -> None:
    tokenizer: JapaneseTransformerTokenizer = (
        JapaneseTransformerTokenizer.from_pretrained("nlp-waseda/roberta-base-japanese")
    )
    lst_tokens_1: List[str] = [
        "▁未来",
        "▁科学",
        "▁部",
        "▁で",
        "▁コンビニ",
        "▁店員",
        "▁に",
        "▁なり",
        "き",
        "って",
        "▁お",
        "釣",
        "り",
        "▁を",
        "▁返し",
        "て",
        "▁いこう",
        "▁!",
    ]
    assert tokenizer.tokenize(sentence_1) == lst_tokens_1
    lst_tokens_2: List[str] = ["▁外国", "▁人", "▁参政", "▁権"]
    assert tokenizer.tokenize(sentence_2) == lst_tokens_2
    lst_tokens_3: List[str] = ["▁魔法", "▁少女", "▁リ", "リ", "カル", "な", "の", "は"]
    assert tokenizer.tokenize(sentence_3) == lst_tokens_3



from typing import Tuple

from src.jptranstokenizer.tokenization_utils import get_word_tokenizer
from src.jptranstokenizer.mainword.base import MainTokenizerABC


def test_get_word_tokenizer() -> None:
    # We test only loading because each tokenize pattern is tested in test_mainword.py
    tpl_word_tokenizer_type: Tuple[str, ...] = [
        "basic",
        "mecab",
        "juman",
        "spacy-luw",
        "sudachi",
    ]
    for word_tokenizer_type in tpl_word_tokenizer_type:
        _: MainTokenizerABC = get_word_tokenizer(
            word_tokenizer_type=word_tokenizer_type, do_lower_case=False
        )



# Copyright 2022 Masahiro Suzuki
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import find_packages, setup


setup(
    name="jptranstokenizer",
    author="Masahiro Suzuki",
    author_email="msuzuki9609@gmail.com",
    description="Japanese Tokenizer with transformers library",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords="NLP japanese transformer transformers MeCab Juman Sudachi spacy",
    license="Apache",
    url="https://github.com/retarfi/jptranstokenizer",
    package_dir={"": "src"},
    packages=find_packages("src"),
    python_requires=">=3.7.0",
    install_requires=["transformers>=4.7.0"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Topic :: Text Processing :: Linguistic",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import re
import unicodedata
from typing import List, Optional

from .base import MainTokenizerABC

try:
    from pyknp import Juman
except ModuleNotFoundError as error:
    raise error.__class__(
        "You need to install pyknp to use JumanTokenizer."
        "See https://github.com/ku-nlp/pyknp for installation."
    )


class JumanTokenizer(MainTokenizerABC):
    def __init__(
        self,
        do_lower_case: bool = False,
        never_split: Optional[List[str]] = None,
        normalize_text: bool = True,
    ):
        super().__init__(do_lower_case=False, never_split=None, normalize_text=True)
        self.juman = Juman()

    def tokenize(self, text: str, never_split: Optional[List[str]] = None) -> List[str]:
        """Tokenizes a piece of text."""
        if self.normalize_text:
            text = unicodedata.normalize("NFKC", text)
        # "#" and "@" at the beginning of a sentence causes timeout error
        text = re.sub("^#", "＃", text)
        text = re.sub("^@", "＠", text)
        never_split = self.never_split + (
            never_split if never_split is not None else []
        )
        tokens = []
        try:
            result = self.juman.analysis(text)
            use_underscore = False
            use_quote = False
        except ValueError:
            # This error is occured because of the Juman's matter about space
            if '"' in text:
                text = text.replace('"', "”")
                use_quote = True
            else:
                use_quote = False
            if re.search(r"\s", text):
                text = re.sub(r"\s", "_", text)
                use_underscore = True
            else:
                use_underscore = False
            try:
                result = self.juman.analysis(text)
            except Exception:
                print(text)
                import sys

                sys.exit(1)
        except Exception:
            print(text)
            import sys

            sys.exit(1)
        for mrph in result:
            token = mrph.midasi
            if self.do_lower_case and token not in never_split:
                token = token.lower()
            tokens.append(token)
        if use_underscore:
            tokens = list(filter(lambda x: x != "_", tokens))
        if use_quote:
            tokens = list(map(lambda x: x.replace("”", '"'), tokens))
        return tokens
